@@ -1,5 +1,5 @@
 // winclip.c (part of mintty)
-// Copyright 2008-23 Andy Koppe, 2018 Thomas Wolff
+// Copyright 2008-23 Andy Koppe, 2018-2025 Thomas Wolff
 // Adapted from code from PuTTY-0.60 by Simon Tatham and team.
 // Licensed under the terms of the GNU General Public License v3 or later.
 
@@ -534,23 +534,26 @@ win_open(wstring wpath, bool adjust_dir)
     }
   }
   wbuf[wl] = 0;
+  //printf("win_open %ls wbuf %ls\n", wpath, wbuf);
   delete(wpath);
 
+  // check for URL
+  wchar *p = wbuf;
+  while (iswalpha(*p)) p++;
+  if (*p == ':' && p - wbuf > 2) {
+    shell_exec(wbuf); // frees wbuf
+    return;
+  }
+
   // guard file opening against foreign network access
   char * buf = cs__wcstoutf(wbuf);
   char * gbuf = guardpath(buf, 4);
+  //printf("win_open buf %s grd %s\n", buf, gbuf);
   free(buf);
   if (!gbuf)
     return;
 
-  wchar *p = wbuf;
-  while (iswalpha(*p)) p++;
-  if (*p == ':' || *wbuf == '\\' || !wcsncasecmp(W("www."), wbuf, 4)) {
-    // Looks like it's a Windows path or URI
-    free(gbuf);
-    shell_exec(wbuf); // frees wbuf
-  }
-  else {
+  {
 #ifdef pathname_conversion_here
 #warning now deprecated; handled via guardpath
     // Need to convert POSIX path to Windows first
@@ -639,6 +642,22 @@ apply_attr_colour_rtf(cattr ca, attr_colour_mode mode, int * pfgi, int * pbgi)
   return ca.attr;
 }
 
+void
+win_copy_text(const char *s)
+{
+  unsigned int size;
+  wchar *text = cs__mbstowcs(s);
+
+  if (text == NULL) {
+    return;
+  }
+  size = wcslen(text);
+  if (size > 0) {
+    win_copy(text, 0, size + 1);
+  }
+  free(text);
+}
+
 void
 win_copy(const wchar *data, cattr *cattrs, int len)
 {
@@ -1040,6 +1059,9 @@ win_copy_as(const wchar *data, cattr *cattrs, int len, char what)
 #endif
 
     CloseClipboard();
+
+    // trigger full selection highlighting
+    term.selection_eq_clipboard = true;
   }
   else {
     GlobalFree(clipdata);
@@ -1328,6 +1350,36 @@ do_win_paste(bool do_path)
   CloseClipboard();
 }
 
+char *
+get_clipboard(void)
+{
+  if (!OpenClipboard(null))
+    return 0;
+
+  HGLOBAL data;
+  char * res;
+  if ((data = GetClipboardData(CF_UNICODETEXT))) {
+    wchar *s = GlobalLock(data);
+    //printf("CF_UNICODETEXT <%ls>\n", s);
+    res = cs__wcstombs(s);
+    GlobalUnlock(data);
+  }
+  else if ((data = GetClipboardData(CF_TEXT))) {
+    char *cs = GlobalLock(data);
+    //printf("CF_TEXT <%s>\n", cs);
+    uint l = MultiByteToWideChar(CP_ACP, 0, cs, -1, 0, 0) - 1;
+    wchar s[l];
+    MultiByteToWideChar(CP_ACP, 0, cs, -1, s, l);
+    res = cs__wcstombs(s);
+    GlobalUnlock(data);
+  }
+  else
+    res = 0;
+
+  CloseClipboard();
+  return res;
+}
+
 void
 win_paste(void)
 {
