@@ -24,6 +24,9 @@ extern HIMC imc;        // the input method context
 extern HWND config_wnd; // the options window
 extern ATOM class_atom;
 
+extern wchar * getregstr(HKEY key, wstring subkey, wstring attribute);
+extern uint getregval(HKEY key, wstring subkey, wstring attribute, uint def);
+
 extern void clear_tabs(void);
 extern void add_tab(uint tabi, HWND wndi);
 extern void win_tab_left(void);
@@ -90,6 +93,7 @@ extern uint dpi;
 extern int per_monitor_dpi_aware;
 extern bool keep_screen_on;
 extern bool force_opaque;
+extern bool checked_desktop_config;
 
 extern bool click_focus_token;
 extern pos last_pos;
@@ -133,8 +137,6 @@ extern void win_show_tip(int x, int y, int cols, int rows);
 extern void win_destroy_tip(void);
 
 extern void taskbar_progress(int percent);
-extern HCURSOR win_get_cursor(bool appmouse);
-extern void set_cursor_style(bool appmouse, wchar * style);
 
 extern void win_init_menus(void);
 extern void win_update_menus(bool callback);
@@ -146,6 +148,10 @@ extern void win_mouse_release(mouse_button, LPARAM);
 extern void win_mouse_wheel(POINT wpos, bool horizontal, int delta);
 extern void win_mouse_move(bool nc, LPARAM);
 
+extern bool is_mouse_mode_by_pixels(void);
+extern HCURSOR win_get_cursor(bool appmouse);
+extern void set_cursor_style(int appmouse, wstring style);
+
 extern mod_keys get_mods(void);
 extern void win_key_reset(void);
 extern void provide_input(wchar);
@@ -177,6 +183,7 @@ extern void win_set_ime_open(bool);
 extern void win_set_ime(bool open);
 extern bool win_get_ime(void);
 
+extern bool is_win_dark_mode(void);
 extern void win_dark_mode(HWND w);
 
 extern void show_message(char * msg, UINT type);
